==> IceWarpServer-9.1.0_RHEL5.tar.gz: html/webdav/inc/xml.php <==
<?php
//Zkladn tdy a funkce pro prci s XML

class xml_tree_node //*
{
  var $_name;
  var $_attribs;
  var $_children;

  function &xml_tree_node($name,$attribs) //*
  {
    $this->_name=$name;
    $this->_attribs=$attribs;
    $this->_children=array();
    return $this;
  }

  function get_name() //*
  {
    return $this->_name;
  }

  function &children() //*
  {
    return $this->_children;
  }

  function children_count() //*
  {
    return count($this->_children);
  }

  function &get_child($name) //*
  {
    $seg=substr($name,0,strcspn($name,"/"));
    if (is_array($this->_children))
      foreach($this->_children as $child)
        if (strcasecmp($child->get_name(),$seg)==0)
          if ($seg==$name)
            return $child;
          else
            return $child->get_child(substr($name,strcspn($name,"/")+1));
    return NULL;
  }

  function &get_data($name) //*
  {
    $tmp=$this->get_child($name);
    if ($tmp)
      return $tmp->_children[0]->get_data();
    else
      return NULL;
  }

  function &add_child(&$xml_node) //*
  {
    return $this->_children[]=&$xml_node;
  }

  function &new_child($name,$attribs) //*
  {
    return $this->add_child(new xml_tree_node($name,$attribs));
  }

  function &new_child_data($name,$attribs,$data) //*
  {
    $tmp=&$this->new_child($name,$attribs);
    $tmp->add_child(new xml_text_node($data));
    return $tmp;
  }

  function print_node($rec_level,$format)
  {
    $result="";
    if ($format)
      for($i=0;$i<$rec_level;$i++) //Pidme mezery podle rovn vnoen tagu
        $result.="  ";

    $result.="<".$this->_name;
    if (is_array($this->_attribs)) //Jestli m tag atributy, tak je vypeme
      foreach($this->_attribs as $key=>$val)
        $result.=" ".$key."=\"".$val."\"";

    if (count($this->_children)) {
      $result.=">";

      $nl=FALSE; //Nov dka
      foreach($this->_children as $child)
      {
        if (!$nl&&(get_class($child)!="xml_text_node")) //Jestli nem tag data, tak oddkujeme
        {
          $nl=TRUE;
          $result.="\r\n";
        }
        $result.=$child->print_node($rec_level+1,$format);
      }

      if ($format&&$nl) //Jestli jsme na nov dce, tak pidme mezery podle rovn vnoen
        for($i=0;$i<$rec_level;$i++)
          $result.="  ";

      $result.="</".$this->_name.">";
    } 
    else 
    { 
      $result.="/>"; 
    }

    $result.="\r\n";
    return $result;
  }
}

class xml_text_node //*
{
  var $_data;

  function &xml_text_node($data) //*
  {
    $this->_data=$data;
    return $this;
  }

  function get_data() //*
  {
    return $this->_data;
  }

  //Polymorfn funkce
  function print_node($rec_level,$format) //*
  {
    return $this->_data;
  }
}

function text_to_xml($xml_text) //*
{
  $xml_parser=xml_parser_create();
  xml_parser_set_option($xml_parser,XML_OPTION_CASE_FOLDING,FALSE);
  if (!xml_parse_into_struct($xml_parser,$xml_text,&$values,&$index))
    die(sprintf("XML error: %s at line %d",xml_error_string(xml_get_error_code($xml_parser)),xml_get_current_line_number($xml_parser)));
  xml_parser_free($xml_parser);

  $stackpos=0;
  $stack=array();
  foreach($values as $val)
  {
    $col=strpos($val['tag'],':');
    if ($col)
      $val['tag']=substr($val['tag'],$col+1);
      
    switch($val["type"])
    {
      case "open":     
        if ($stackpos)
          $stack[$stackpos]=&$stack[$stackpos-1]->new_child($val["tag"],$val["attributes"]);
        else
          $stack[0]=&new xml_tree_node($val["tag"],$val["attributes"]);
        $stackpos++;
      break;

      case "complete":
        if ($stackpos)
          $stack[$stackpos]=&$stack[$stackpos-1]->new_child($val["tag"],$val["attributes"]);
        else
          $stack[0]=&new xml_tree_node($val["tag"],$val["attributes"]);
        if (isset($val['value']))   
          $stack[$stackpos]->add_child(new xml_text_node($val['value']));
      break;

      case "close":
        if ($stackpos)
          $stackpos--;
    }
  }
  return $stack[0];
}
?>

==> IceWarpServer-9.4.1_RHEL5.tar.gz: html/webdav/inc/xml.php <==
<?php
//Zkladn tdy a funkce pro prci s XML

class xml_tree_node
{
  var $_name;
  var $_attribs;
  var $_children;

  function &xml_tree_node($name,$attribs)
  {
    $this->_name=$name;
    $this->_attribs=$attribs;
    $this->_children=array();
    return $this;
  }

  function get_name()
  {
    return $this->_name;
  }

  function &children()
  {
    return $this->_children;
  }

  function children_count()
  {
    return count($this->_children);
  }

  function &get_child($name)
  {
    $seg=substr($name,0,strcspn($name,"/"));
    if (is_array($this->_children))
      foreach($this->_children as $child)
        if (strcasecmp($child->get_name(),$seg)==0)
          if ($seg==$name)
            return $child;
          else
            return $child->get_child(substr($name,strcspn($name,"/")+1));
    return NULL;
  }

  function &get_data($name)
  {
    $tmp=$this->get_child($name);
    if ($tmp)
      return $tmp->_children[0]->get_data();
    else
      return NULL;
  }

  function &add_child(&$xml_node)
  {
    return $this->_children[]=&$xml_node;
  }

  function &new_child($name,$attribs)
  {
    return $this->add_child(new xml_tree_node($name,$attribs));
  }

  function &new_child_data($name,$attribs,$data,$encode=true)
  {
    $tmp=&$this->new_child($name,$attribs);
    $tmp->add_child(new xml_text_node($data,$encode));
    return $tmp;
  }

  function print_node($rec_level,$format)
  {
    $result="";
    if ($format)
      for($i=0;$i<$rec_level;$i++) //Pidme mezery podle rovn vnoen tagu
        $result.="  ";

    $result.="<".$this->_name;
    if (is_array($this->_attribs)) //Jestli m tag atributy, tak je vypeme
      foreach($this->_attribs as $key=>$val)
        $result.=" ".$key."=\"".$val."\"";

    if (count($this->_children)) {
      $result.=">";

      $nl=false; //Nov dka
      foreach($this->_children as $child)
      {
        if (!$nl&&(get_class($child)!="xml_text_node")) //Jestli nem tag data, tak oddkujeme
        {
          $nl=true;
          $result.="\r\n";
        }
        $result.=$child->print_node($rec_level+1,$format);
      }

      if ($format&&$nl) //Jestli jsme na nov dce, tak pidme mezery podle rovn vnoen
        for($i=0;$i<$rec_level;$i++)
          $result.="  ";

      $result.="</".$this->_name.">";
    } 
    else 
      $result.="/>"; 

    $result.="\r\n";
    return $result;
  }
}

class xml_text_node
{
  var $_data;

  function &xml_text_node($data,$encode=true)
  {
    if ($encode)
    	$this->_data=htmlspecialchars($data);
		else
			$this->_data=$data;
			
		return $this;
  }

  function get_data()
  {
    return $this->_data;
  }

  //Polymorfn funkce
  function print_node($rec_level,$format)
  {
    return $this->_data;
  }
}

function text_to_xml($xml_text)
{
  $xml_parser=xml_parser_create();
  xml_parser_set_option($xml_parser,XML_OPTION_CASE_FOLDING,FALSE);
  if (!xml_parse_into_struct($xml_parser,$xml_text,&$values,&$index))
    die(sprintf("XML error: %s at line %d",xml_error_string(xml_get_error_code($xml_parser)),xml_get_current_line_number($xml_parser)));
  xml_parser_free($xml_parser);

  $stackpos=0;
  $stack=array();
  foreach($values as $val)
  {
    $col=strpos($val['tag'],':');
    if ($col)
      $val['tag']=substr($val['tag'],$col+1);
      
    switch($val["type"])
    {
      case "open":     
        if ($stackpos)
          $stack[$stackpos]=&$stack[$stackpos-1]->new_child($val["tag"],$val["attributes"]);
        else
          $stack[0]=&new xml_tree_node($val["tag"],$val["attributes"]);
        $stackpos++;
      break;

      case "complete":
        if ($stackpos)
          $stack[$stackpos]=&$stack[$stackpos-1]->new_child($val["tag"],$val["attributes"]);
        else
          $stack[0]=&new xml_tree_node($val["tag"],$val["attributes"]);
        if (isset($val['value']))   
          $stack[$stackpos]->add_child(new xml_text_node($val['value']));
      break;

      case "close":
        if ($stackpos)
          $stackpos--;
    }
  }
  return $stack[0];
}
?>
